@@ -3,7 +3,7 @@ package Set::Similarity;
 use strict;
 use warnings;
 
-our $VERSION = 0.003;
+our $VERSION = 0.007;
 
 sub new {
   my $class = shift;
@@ -59,19 +59,18 @@ sub from_tokens {
   my $self = shift;
   my $tokens1 = shift || [];
   my $tokens2 = shift || [];
-  
+
   $tokens1 = ref $tokens1 ? $tokens1 : [$tokens1];
   $tokens2 = ref $tokens2 ? $tokens2 : [$tokens2];
-  
+
   # uncoverable condition false
   return 1 if (!scalar @$tokens1 && !scalar @$tokens2);
   return 0 unless (scalar @$tokens1 && scalar @$tokens2 );
-  
-  my %unique1;
-  @unique1{@$tokens1} = ();
-  my %unique2;
-  @unique2{@$tokens2} = ();
-  return $self->from_sets(\%unique1,\%unique2);
+    
+  return $self->from_sets(
+    [$self->uniq($tokens1)],
+    [$self->uniq($tokens2)],
+  );
 }
 
 # overlap is default
@@ -85,16 +84,24 @@ sub from_sets {
 }
 
 sub intersection { 
-  scalar grep { exists ${$_[1]}{$_} } keys %{$_[2]}; 
+  my %uniq;
+  @uniq{@{$_[1]}} = ();
+  scalar grep { exists $uniq{$_} } @{$_[2]};
+}
+
+sub uniq {
+  my %uniq; 
+  @uniq{@{$_[1]}} = ();
+  return keys %uniq; 
 }
 
 sub combined_length {
-  scalar(keys %{$_[1]}) + scalar(keys %{$_[2]});
+  scalar(@{$_[1]}) + scalar(@{$_[2]});
 }
 
 sub min {
-  (scalar(keys %{$_[1]}) < scalar(keys %{$_[2]}))
-    ? scalar(keys %{$_[1]}) : scalar(keys %{$_[2]});
+  (scalar(@{$_[1]}) < scalar(@{$_[2]}))
+    ? scalar(@{$_[1]}) : scalar(@{$_[2]});
 }
 
 1;
